import { HttpClient } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs';
import { Employer } from '../../models/employer.model';
import { EspaceCoopCNFS } from '../../models/espaceCoopCNFS.model';
import { Job } from '../../models/job.model';
import { JobGroup } from '../../models/jobGroup.model';
import { Structure } from '../../models/structure.model';
import { User } from '../../models/user.model';
import { AdminStructureList } from '../models/adminStructureList.interface';
import { StructureAdminInfo } from '../models/demandAttachment.model';

@Injectable({
  providedIn: 'root',
})
export class AdminService {
  private readonly baseUrl = 'api/admin';
  private readonly jobBaseUrl = 'api/jobs';
  private readonly employerBaseUrl = 'api/employer';
  private readonly jobsGroupBaseUrl = 'api/jobs-groups';
  constructor(private http: HttpClient) {}

  // Return pendingAttachments of all profiles.
  public getPendingStructure(): Observable<StructureAdminInfo[]> {
    return this.http.get<StructureAdminInfo[]>(`${this.baseUrl}/pendingStructures`);
  }

  public getAllStructureAdmin(): Observable<AdminStructureList> {
    return this.http.get<AdminStructureList>(`${this.baseUrl}/adminStructuresList`);
  }

  public getUsers(): Observable<User[]> {
    return this.http.get<User[]>(`${this.baseUrl}/searchUsers`);
  }

  public getAttachedUsers(): Observable<User[]> {
    return this.http.get<User[]>(`${this.baseUrl}/attachedUsers`);
  }

  public getUnAttachedUsers(): Observable<User[]> {
    return this.http.get<User[]>(`${this.baseUrl}/unAttachedUsers`);
  }

  public getUnVerifiedUsers(): Observable<User[]> {
    return this.http.get<User[]>(`${this.baseUrl}/unVerifiedUsers`);
  }

  public setUserJob(jobId: string, userId: string): Observable<User> {
    return this.http.put<User>(`${this.baseUrl}/setUserJob`, {
      userId,
      jobId,
    });
  }

  public setUserEmployer(employerId: string, userId: string): Observable<User> {
    return this.http.put<User>(`${this.baseUrl}/setUserEmployer`, {
      userId,
      employerId,
    });
  }

  // By default return only validated jobs
  public getJobs(): Observable<Job[]> {
    return this.http.get<Job[]>(`api/jobs`);
  }

  // By default return only validated employers
  public getEmployers(): Observable<Employer[]> {
    return this.http.get<Employer[]>(`api/employer`);
  }

  // populated with users for admin purposes
  public getUnvalidatedJobs(): Observable<Job[]> {
    return this.http.get<Job[]>(`${this.jobBaseUrl}/unvalidated`);
  }

  // populated with users for admin purposes
  public getValidatedJobs(): Observable<Job[]> {
    return this.http.get<Job[]>(`${this.jobBaseUrl}/validated`);
  }

  // populate with jobs groups for admin purposes
  public getJobsGroups(): Observable<JobGroup[]> {
    return this.http.get<JobGroup[]>(`${this.jobsGroupBaseUrl}`);
  }

  public getJobsGroupsWithJobs(): Observable<JobGroup[]> {
    return this.http.get<JobGroup[]>(`${this.jobsGroupBaseUrl}/with-jobs`);
  }

  public createJobsGroup(name: string): Observable<JobGroup> {
    return this.http.post<JobGroup>(`${this.jobsGroupBaseUrl}`, { name });
  }

  public deleteJobsGroup(id: string): Observable<JobGroup> {
    return this.http.delete<JobGroup>(`${this.jobsGroupBaseUrl}/${id}`);
  }

  public editJobsGroup(id: string, name: string): Observable<JobGroup> {
    return this.http.put<Job>(`${this.jobsGroupBaseUrl}/${id}/`, { name });
  }

  public getUnvalidatedEmployers(): Observable<Employer[]> {
    return this.http.get<Employer[]>(`${this.employerBaseUrl}/unvalidated`);
  }

  public getValidatedEmployers(): Observable<Employer[]> {
    return this.http.get<Employer[]>(`${this.employerBaseUrl}/validated`);
  }

  public searchUsers(searchString: string): Observable<User[]> {
    return this.http.post<User[]>(`${this.baseUrl}/searchUsers`, { searchString });
  }

  public deleteUser(id: string): Observable<User> {
    return this.http.delete<User>(`${this.baseUrl}/user/` + id);
  }

  public deleteJob(id: string): Observable<Job> {
    return this.http.delete<Job>(`${this.jobBaseUrl}/${id}`);
  }

  public deleteEmployer(id: string): Observable<Employer> {
    return this.http.delete<Employer>(`${this.employerBaseUrl}/${id}`);
  }

  public createJob(name: string): Observable<Job> {
    return this.http.post<Job>(`${this.jobBaseUrl}`, { name, hasPersonalOffer: true });
  }

  public updateJob(id: string, job: Job): Observable<Job> {
    return this.http.put<Job>(`${this.jobBaseUrl}/${id}`, job);
  }

  public createEmployer(name: string): Observable<Employer> {
    return this.http.post<Employer>(`${this.employerBaseUrl}`, { name });
  }

  public validateJob(id: string): Observable<Job> {
    return this.http.post<Job>(`${this.jobBaseUrl}/validate/${id}`, {});
  }

  public validateEmployer(id: string): Observable<Employer> {
    return this.http.post<Employer>(`${this.employerBaseUrl}/validate/${id}`, {});
  }

  public editJob(id: string, name: string, hasPersonalOffer: boolean): Observable<Job> {
    return this.http.put<Job>(`${this.jobBaseUrl}/${id}`, {
      name,
      hasPersonalOffer,
    });
  }

  public editEmployer(id: string, name: string): Observable<Employer> {
    return this.http.put<Employer>(`${this.employerBaseUrl}/${id}`, {
      name,
    });
  }

  public mergeEmployer(sourceEmployerId: string, targetEmployerId: string): Observable<Employer> {
    return this.http.put<Employer>(`${this.employerBaseUrl}/merge`, {
      sourceEmployerId,
      targetEmployerId,
    });
  }

  public mergeJob(sourceJobId: string, targetJobId: string): Observable<Job> {
    return this.http.put<Job>(`${this.jobBaseUrl}/merge`, {
      sourceJobId,
      targetJobId,
    });
  }

  public acceptStructureClaim(
    userEmail: string,
    structureId: number,
    structureName: string
  ): Observable<StructureAdminInfo[]> {
    return this.http.post<StructureAdminInfo[]>(`${this.baseUrl}/validatePendingStructure`, {
      userEmail,
      structureId,
      structureName,
    });
  }

  public refuseStructureClaim(
    userEmail: string,
    structureId: number,
    structureName: string
  ): Observable<StructureAdminInfo[]> {
    return this.http.post<StructureAdminInfo[]>(`${this.baseUrl}/rejectPendingStructure`, {
      userEmail,
      structureId,
      structureName,
    });
  }

  public getDeletedStructures(): Observable<Structure[]> {
    return this.http.get<Structure[]>(`${this.baseUrl}/getDeletedStructures`);
  }

  public restoreDeletedStructure(structureId: string): Observable<void> {
    return this.http.post<void>(`${this.baseUrl}/restoreDeletedStructure/${structureId}`, {});
  }

  public getAllEspaceCoopCNFS() {
    return this.http.get<EspaceCoopCNFS[]>(`${this.baseUrl}/espaceCoopCNFS`);
  }

  public getAllResinCNFS() {
    return this.http.get<User[]>(`${this.baseUrl}/resinCNFS`);
  }
}
