import { AfterContentChecked, ChangeDetectorRef, Component, OnInit } from '@angular/core';
import { FormGroup, UntypedFormGroup } from '@angular/forms';
import { User } from '../../models/user.model';
import { ProfileService } from '../../profile/services/profile.service';
import { AuthService } from '../../services/auth.service';
import { NotificationService } from '../../services/notification.service';
import { OnlineMediationService } from '../../services/online-mediation.service';
import { StructureService } from '../../services/structure.service';
import { CategoryEnum } from '../../shared/enum/category.enum';
import { Category } from '../../structure-list/models/category.model';
import { Filter } from '../../structure-list/models/filter.model';
import { Module } from '../../structure-list/models/module.model';
import { SearchService } from '../../structure-list/services/search.service';
import { OrientationUtils } from '../../utils/orientationUtils';
import { Utils } from '../../utils/utils';
import { MeetingRDVSSteps } from './enums/MeetingRDVSSteps.enum';
import { FiltersSteps } from './enums/filtersSteps.enum';
import { GenericOrientationSteps } from './enums/genericOrientationSteps.enum';
import { HotlineMediationSteps } from './enums/hotlineMediationSteps.enum';
import { NeedsType } from './enums/needs.enum';
import { OnlineDemarche } from './enums/onlineDemarche.enum';
import { OnlineDemarchesCommonSteps } from './enums/onlineDemarchesCommonSteps.enum';
import { StructuresListSteps } from './enums/structuresListSteps.enum';
import { FiltersForm } from './interfaces/filtersForm.interface';
import { IOnlineMediation } from './interfaces/onlineMediation.interface';
import { StructureOrientationForm } from './interfaces/structureOrientationForm.interface';
import { StructureOrientator } from './interfaces/structureOrientator.interface';
import { MediationType } from './types/mediation.type';
import { MediationStepType } from './types/mediationStep.type';

@Component({
  selector: 'app-orientation-form-view',
  templateUrl: './orientation-form-view.component.html',
  styleUrls: ['./orientation-form-view.component.scss'],
})
export class OrientationFormViewComponent implements OnInit, AfterContentChecked {
  public static readonly MAX_STEP =
    (Math.max(
      Object.keys(HotlineMediationSteps).length,
      Object.keys(MeetingRDVSSteps).length,
      Object.keys(StructuresListSteps).length
    ) +
      Object.keys(OnlineDemarchesCommonSteps).length) /
    2;
  public orientationUtils = new OrientationUtils();
  public utils = new Utils();
  // OnlineDemarches
  public onlineDemarcheForm: UntypedFormGroup;
  public onlineDemarcheType: OnlineDemarche;
  // Equipments
  public filtersForm: FormGroup<FiltersForm>;
  public structureOrientationForm: FormGroup<StructureOrientationForm>;
  // Orientator
  public orientator: FormGroup<StructureOrientator>;
  // Enum
  public needEnum = NeedsType;
  // Global
  public needType: NeedsType;
  public previousNeedType: NeedsType;
  public currentType: MediationType = OnlineDemarche.common;
  public currentStep: MediationStepType = null;
  public isLastStep = false;
  public isRdvsWindowStep = false;
  public hideNavButtons = false;
  public nbSteps: number = OrientationFormViewComponent.MAX_STEP;
  public isPageValid: boolean = false;
  public filters: Filter[] = [];
  public profile: User;
  public canDeactivate: boolean = false;
  public showLoginModal: boolean = false;
  public fullScreen: boolean = false;
  public shouldResetOrientation = false;
  // Categories
  public categories: { [key: string]: Category } = {
    age: {
      id: '',
      name: '',
      theme: '',
      modules: [],
    },
    languageAndIlliteracy: {
      id: '',
      name: '',
      theme: '',
      modules: [],
    },
    genre: {
      id: '',
      name: '',
      theme: '',
      modules: [],
    },
    handicaps: {
      id: '',
      name: '',
      theme: '',
      modules: [],
    },
  };

  // Modal canExit var
  public showConfirmationModal = false;
  private resolve: Function;

  constructor(
    private onlineMediationService: OnlineMediationService,
    private notificationService: NotificationService,
    private authService: AuthService,
    private profileService: ProfileService,
    private searchService: SearchService,
    private structureService: StructureService,
    private cdref: ChangeDetectorRef
  ) {
    this.setCategories();
  }
  async ngOnInit() {
    if (!this.authService.isLoggedIn()) {
      this.showLoginModal = true;
    } else {
      this.profile = await this.profileService.getProfile();

      if (this.profile?.structuresLink?.length === 1) {
        this.structureService.getStructure(this.profile.structuresLink[0]).subscribe(
          (_structure) =>
            (this.orientator = this.orientationUtils.createStructureOrientatorForm({
              structureName: _structure.structureName,
              structureMail: _structure.contactMail,
              structurePhone: _structure.contactPhone,
            }))
        );
      }
    }

    if (history.state.mediationNumerique) {
      this.currentStep = 0;
      this.needType = this.needEnum.learnSkills;
      this.setCurrentNeedType(NeedsType.learnSkills);
    }
  }

  ngAfterContentChecked() {
    this.cdref.detectChanges();
  }

  public validatePage(event: boolean) {
    this.isPageValid = event;
  }

  public setStepNumber(enumLength: number): void {
    this.nbSteps = enumLength / 2;
  }

  public setCurrentNeedType(needType: NeedsType): void {
    this.needType = needType;
    this.validatePage(true);
    // Init form according to needType
    switch (needType) {
      case NeedsType.onlineDemarch:
        this.onlineDemarcheForm = this.orientationUtils.createOnlineDemarchesForm();
        break;
      case NeedsType.equipmentAccess:
      case NeedsType.equipmentBuy:
        this.filtersForm = this.orientationUtils.createFiltersForm();
        break;
      case NeedsType.learnSkills:
        this.filtersForm = this.orientationUtils.createFiltersForm();
        this.onlineDemarcheForm = this.orientationUtils.createOnlineDemarchesForm();
        break;
      default:
        throw new Error(`Not implemented NeedsType in OrientationFormViewComponent ${needType}`);
    }
  }

  public checkLastStep(): void {
    if (
      this.needType === NeedsType.onlineDemarch &&
      this.currentType === OnlineDemarche.meetings &&
      this.currentStep === MeetingRDVSSteps.location
    ) {
      this.isRdvsWindowStep = true;
    }
    if (
      this.needType === NeedsType.onlineDemarch &&
      this.currentType === OnlineDemarche.onlineMediation &&
      this.currentStep === HotlineMediationSteps.orientationRecap
    ) {
      this.isLastStep = true;
    } else if (this.isStructureListForm() && this.currentStep === StructuresListSteps.orientationRecap) {
      this.isLastStep = true;
    } else this.isLastStep = false;
  }

  /**
   * Handle online mediation finish by sending appointements to server
   */
  public handleOnlineAppointment(): void {
    const toCreate: IOnlineMediation = {
      ...this.onlineDemarcheForm.value,
      onlineDemarchType: this.onlineDemarcheForm.value.filters.map((module: Module) => module.displayText),
    };
    this.onlineMediationService.createOnlineMediation(toCreate).subscribe((data) => {
      if (data) {
        this.canDeactivate = true;
        this.notificationService.showSuccess('Votre démarche en ligne a bien été enregistrée');
      } else {
        this.notificationService.showError('Echec de la création de votre démarche en ligne');
      }
      this.printForm();
    });
  }

  public printForm(): void {
    window.print();
    setTimeout(() => {
      this.canDeactivate = true;
      window.location.reload();
    }, 100);
  }

  public setOnlineEquipmentsForm() {
    this.currentType = OnlineDemarche.structureList;
    this.currentStep = StructuresListSteps.pmrAccess;
    // first filter screen + regular StructureList form
    this.setStepNumber(Object.keys(StructuresListSteps).length);
    this.structureOrientationForm = this.orientationUtils.createStructureOrientationForm(
      this.filtersForm.value.filters,
      this.orientator
    );
  }

  public setOnlineProceduresForm() {
    switch (this.onlineDemarcheForm.get('accompanimentType').value) {
      case OnlineDemarche.structureList:
        this.currentType = OnlineDemarche.structureList;
        this.currentStep = StructuresListSteps.pmrAccess;
        this.setStepNumber(Object.keys(StructuresListSteps).length);
        this.onlineDemarcheForm = this.orientationUtils.createStructureOrientationForm(
          this.utils.convertFiltersToModule(this.filters)
        );
        break;
      case OnlineDemarche.meetings:
        this.currentType = OnlineDemarche.meetings;
        this.currentStep = MeetingRDVSSteps.infoScreen;
        this.setStepNumber(Object.keys(MeetingRDVSSteps).length);
        this.skipStructureOrientator();
        this.onlineDemarcheForm = this.orientationUtils.createMeetingRDVSForm(this.filters, this.orientator);
        break;
      case OnlineDemarche.onlineMediation:
        this.currentType = OnlineDemarche.onlineMediation;
        this.currentStep = HotlineMediationSteps.infoScreen;
        this.setStepNumber(Object.keys(HotlineMediationSteps).length);
        this.onlineDemarcheForm = this.orientationUtils.createOnlineMediationForm(
          this.utils.convertFiltersToModule(this.filters)
        );
        break;
      default:
        throw new Error('Not implemented value in nextPage()');
    }
  }

  private async setCategories(): Promise<void> {
    const categories = await this.searchService.getCategories().toPromise();
    categories.forEach((categ) => {
      switch (categ.id) {
        case CategoryEnum.age: {
          this.categories.age = categ;
          break;
        }
        case CategoryEnum.languageAndIlliteracy: {
          this.categories.languageAndIlliteracy = categ;
          break;
        }
        case CategoryEnum.genre: {
          this.categories.genre = categ;
          break;
        }
        case CategoryEnum.handicaps: {
          this.categories.handicaps = categ;
          break;
        }
      }
    });
  }

  public skipStructureOrientator() {
    if (this.profile?.structuresLink?.length === 1) {
      // skip first step
      this.currentStep++;
    }
  }

  public setOnlineProcedureFilters(): void {
    // Handle online procedures
    const onlineProcedures: Filter[] = this.onlineDemarcheForm.value.onlineDemarcheType.map((module: Module) => {
      return new Filter('onlineProcedures', module.id, module.displayText);
    });
    if (onlineProcedures.length > 0) this.filters = [...onlineProcedures, ...this.filters];
  }

  public setFilters(category: string): void {
    // Handle online procedures
    this.filters = this.filtersForm.value.filters.map((module: Module) => {
      return new Filter(category, module.id, module.displayText);
    });
  }

  public nextPage(): void {
    this.isPageValid = false;
    this.isRdvsWindowStep = false;
    this.hideNavButtons = false;
    // Handle filters form
    if (
      this.isFilterForm() &&
      this.currentType === GenericOrientationSteps.common &&
      this.currentStep === FiltersSteps.filterChoice
    ) {
      switch (this.needType) {
        case NeedsType.equipmentAccess:
          this.previousNeedType = NeedsType.equipmentAccess;
          this.setFilters('selfServiceMaterial');
          // Switch to structure list form
          this.setOnlineEquipmentsForm();
          break;
        case NeedsType.equipmentBuy:
          this.previousNeedType = NeedsType.equipmentBuy;
          this.setFilters('solidarityMaterial');
          // Switch to structure list form
          this.setOnlineEquipmentsForm();
          break;
        case NeedsType.learnSkills:
          this.previousNeedType = NeedsType.learnSkills;
          this.setFilters('baseSkills');
          this.needType = NeedsType.onlineDemarch;
          this.currentStep = OnlineDemarchesCommonSteps.accompanimentType;
          break;
      }
      return;
    }
    // Online demarches
    if (this.needType === NeedsType.onlineDemarch) {
      if (
        this.currentType === OnlineDemarche.common &&
        this.currentStep === OnlineDemarchesCommonSteps.accompanimentType
      ) {
        this.setOnlineProcedureFilters();
        this.setOnlineProceduresForm();
        this.previousNeedType = NeedsType.onlineDemarch;
        return;
      }
      // Handle last screen Online appointment and print
      if (
        this.currentType === OnlineDemarche.onlineMediation &&
        this.currentStep === HotlineMediationSteps.orientationRecap
      ) {
        this.handleOnlineAppointment();
        return;
      }
    }

    // Structure list handling
    if (this.isStructureListForm()) {
      // Print last screen
      if (this.currentStep === StructuresListSteps.orientationRecap) {
        this.printForm();
        return;
      }
      // Carto display case for full screen
      // Set to full screen
      if (this.currentStep === StructuresListSteps.address) {
        this.fullScreen = true;
      }
      // Unset fullscreen
      if (this.currentStep === StructuresListSteps.structureChoice) {
        this.fullScreen = false;
        this.skipStructureOrientator();
      }
    }

    // Default case
    if (this.currentStep === null) {
      // handle the form start
      this.currentStep = 0;
      return;
    }
    if (this.currentStep < this.nbSteps) {
      this.currentStep++;
      this.checkLastStep();
      this.checkHideNavButtons();
    }
  }

  public prevPage(): void {
    this.isRdvsWindowStep = false;
    this.hideNavButtons = false;
    // Structure list previous page
    if (this.isStructureListForm()) {
      if (this.currentStep === StructuresListSteps.structureChoice) {
        this.fullScreen = false;
      }
      if (this.currentStep === StructuresListSteps.structureOrientator) {
        this.fullScreen = true;
      }
    }

    // Default case for first form step. If there was a previous form (like filterForm) we go back to this form
    if (this.currentStep === 0) {
      switch (this.previousNeedType) {
        case NeedsType.onlineDemarch:
          this.currentType = OnlineDemarche.common;
          this.currentStep = OnlineDemarchesCommonSteps.accompanimentType;
          this.onlineDemarcheForm = this.orientationUtils.createOnlineDemarchesForm();
          this.previousNeedType = null;
          return;
        case NeedsType.learnSkills:
        case NeedsType.equipmentAccess:
        case NeedsType.equipmentBuy:
          this.needType = this.previousNeedType;
          this.currentType = OnlineDemarche.common;
          this.currentStep = OnlineDemarchesCommonSteps.accompanimentType;
          this.previousNeedType = null;
          break;
        default:
          // Last step, reset all
          this.reset();
          return;
      }
      // return;
      // handle the form start
    }
    if (this.currentStep > 0) {
      // Special case for learnSkills
      if (this.previousNeedType === NeedsType.learnSkills && this.currentStep === 1) {
        this.needType = this.previousNeedType;
        this.currentType = OnlineDemarche.common;
        this.currentStep = OnlineDemarchesCommonSteps.accompanimentType;
        this.previousNeedType = null;
      }
      this.currentStep--;
      this.checkLastStep();
      this.checkHideNavButtons();
    }
  }

  public reset(): void {
    this.onlineDemarcheForm = undefined;
    this.onlineDemarcheType = undefined;
    this.filtersForm = undefined;
    this.currentStep = null;
    this.currentType = OnlineDemarche.common;
    this.nbSteps = OrientationFormViewComponent.MAX_STEP;
    this.isPageValid = false;
    this.needType = undefined;
    this.filters = [];
  }

  /**
   * Return if it's a Structure List form
   * @returns
   */
  private isStructureListForm(): boolean {
    return (
      (this.needType === NeedsType.onlineDemarch ||
        this.needType === NeedsType.equipmentAccess ||
        this.needType === NeedsType.equipmentBuy) &&
      this.currentType === OnlineDemarche.structureList
    );
  }
  /**
   * Return if it's a filter form
   * @returns
   */
  private isFilterForm(): boolean {
    return (
      this.needType === NeedsType.equipmentAccess ||
      this.needType === NeedsType.equipmentBuy ||
      this.needType === NeedsType.learnSkills
    );
  }

  public canExit(): Promise<boolean> {
    // Avoid confirmation when user submit form and leave.
    if (this.canDeactivate || this.currentStep === null) {
      return new Promise((resolve) => resolve(true));
    } else {
      return new Promise((resolve) => this.showModal(resolve));
    }
  }
  private showModal(resolve: Function): void {
    this.showConfirmationModal = true;
    this.resolve = resolve;
  }
  public hasRedirectionAccepted(hasAccept: boolean): void {
    this.resolve(hasAccept);
    this.showConfirmationModal = false;
  }
  public checkHideNavButtons(): void {
    if (
      this.needType === NeedsType.onlineDemarch &&
      this.currentType === OnlineDemarche.meetings &&
      this.currentStep === MeetingRDVSSteps.rdvsDoing
    ) {
      this.hideNavButtons = true;
    }
  }
  public setResetOrientation(): void {
    this.shouldResetOrientation = true;
  }
}
